/*
 * Simple test application to test usage of the Xc Widget Set.
 */

#include <stdio.h>

/* Xlib headers */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>

/* Standard Xt headers */
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/* Athena Widget Set headers */
#ifdef X11R3
#include <Form.h>
#else
#include <X11/Xaw/Form.h>
#endif

/* Control Panel Widget Set headers */
#include <X11/Xc/Xc.h>
#include <X11/Xc/VButton.h>
#include <X11/Xc/SimpleB.h>
#include <X11/Xc/Slider.h>
#include <X11/Xc/Lcd.h>
#include <X11/Xc/BarGraf.h>
#include <X11/Xc/ToggleB.h>
#include <X11/Xc/Led.h>


/* Widget IDs */
Widget TopLevel, 
	MainBox, Panel1, Panel2, 
	SimpleButton1, SimpleButton2,
	VButton1, VButton2, 
	Slider1, Slider2,
	Lcd1, Lcd2,
	BarGraph1,BarGraph2,
	Led1, Led2, Led3,
	Toggle1, Toggle2;



/* Callback for the Quit button: simply quits the application. */
void Quit(w, client_data, call_data)
Widget w;
caddr_t client_data, call_data;
{
   XtDestroyWidget(TopLevel);
   exit(0);
}


/* Callback for the Reset button: resets Lcd1's counter value to 0. */
void Reset_lcd(w, lcd, call_data)
Widget w;
Widget lcd; 
caddr_t call_data;
{
XcVType value;

   value.lval = 0L;
   XcLcdUpdateValue(lcd, &value);

}



/*
 * Simple callback for the Value style widgets. It just prints the current
 * value of the particular widget based on it's datatype.
 */
void Val_callback(w, client_data, call_data)
Widget w;
caddr_t client_data;
XcData call_data;
{
XcVType new_value;
 
   if (call_data->dtype == XcLval)
      fprintf(stderr, "New Value: %ld\n", call_data->value.lval);
   if (call_data->dtype == XcHval)
      fprintf(stderr, "New Value: %lX\n", call_data->value.lval);
   else if (call_data->dtype == XcFval)
   {
      fprintf(stderr, "New Value: %.*f\n", call_data->decimals, 
						call_data->value.fval);
      new_value.fval = call_data->value.fval;
   }
}



/*
 * VButton2's callback. It checks the ValueButton's current value and sets
 * Led1's color and state string based on that value (i.e., indicates the
 * threshold of the voltage setting).
 */
typedef struct thresh_s
{
   float lower;
   float upper;
   char msg[25];
   char color[25];
} Threshold_struct;

void Set_threshold(w, led, call_data)
Widget w;
LedButtonWidget led;
XcData call_data;
{
/* Local variables */
int j, idx = 0;
static Threshold_struct threshold[6] =
{
   { -15.0, -10.0, "Critical", "red" },
   { -9.9, -5.0, "Warning", "yellow" },
   { -4.9, 0.0, "Safe", "green" },
   { 0.0, 4.9, "Safe", "green" },
   { 5.0, 9.9, "Warning", "yellow" },
   { 10.0, 15.0, "Critical", "red" }
};
 
   for (j = 0; j < XtNumber(threshold); j++)
      if ((call_data->value.fval <= threshold[j].upper) &&
          (call_data->value.fval >= threshold[j].lower))
      {
	 idx = j;
	 break;
      }

/* Now set the threshold indicator based on the value. */
   XcLedSetState(led, threshold[idx].color, threshold[idx].msg);

} /* end of Set_threshold */



/*
 * Slider1's callback. It checks to see if Toggle1 is On, and if it is,
 * it updates BarGraph1 with the new value.
 */
void Slider1_callback(w, bargraph, call_data)
Widget w;
Widget bargraph;
XcData call_data;
{
/* Local variables */
XcToggleState state;

   XcToggleGet(Toggle1, &state); 
   if (state == XcOn)
      XcBGUpdateValue(bargraph, &call_data->value);

}


/*
 * Slider2's callback. It checks to see if Toggle2 is On, and if it is,
 * it updates Lcd2 with the new value.
 */
void Slider2_callback(w, lcd, call_data)
Widget w;
Widget lcd;
XcData call_data;
{
XcToggleState state;
 
   XcToggleGet(Toggle2, &state); 
   if (state == XcOn)
      XcLcdUpdateValue(lcd, &call_data->value);

}



/*
 * updateCallback for the Lcd1 widget.  Simply increments the current value
 * based on the dataType.  Used to implement a simple count-up counter.
 * It also updates BarGraph2 with Lcd1's new value.
 */
void Lcd1_callback(w, bargraph, call_data)
Widget w;
Widget bargraph;
XcData call_data;
{

   call_data->value.lval++; 

/* Update the BarGraph widget also. */
   XcBGUpdateValue(bargraph, &call_data->value);

}



/*
 * updateCallback for the Led2 widget.  Simply cycles the Led's color 
 * between red and black every second.
 */
void Led2_callback(w, client_data, call_data)
Widget w;
caddr_t client_data;
XcLedCallData call_data;
{
static char current_led_color[10] = "black";

   if (strcmp(current_led_color, "red") == 0)
      strcpy(current_led_color, "black");
   else
      strcpy(current_led_color, "red");

   strcpy(call_data->color, current_led_color);

}


/*
 * updateCallback for the Led3 widget.  It updates the Led's state string
 * every 5 seconds from a small set of states.
 */
void Led3_callback(w, client_data, call_data)
Widget w;
caddr_t client_data;
XcLedCallData call_data;
{
static cur_index = 0;
static char *states[4] = { "Booting", "Halted", "Running", "Testing" };

   if (cur_index == 3)
      cur_index = 0;
   else
      cur_index++;

   strcpy(call_data->str, states[cur_index]);

}





/*********************************************************************
 * Function main.
 *********************************************************************/
main (argc, argv)
int argc;
char *argv[];
{
/* Local variables */
int i = 0;
unsigned long TheForeground, TheBackground;
Arg args[10];
XcVType up_bound, lo_bound, inc, test;
Display *display;
int scr_num;
XColor col, exact;
XFontStruct *font;


/* Initialize the toolkit and TopLevel window. */
   TopLevel = XtInitialize(argv[0], "Xctest", NULL, 0, &argc, argv);

   display = XtDisplay(TopLevel);
   scr_num = DefaultScreen(display);

/*
 * Create the main form widgets to manage all of the other application 
 * widgets.
 */
   MainBox = XtCreateManagedWidget("mainbox", formWidgetClass,
					TopLevel, NULL, 0);
   Panel1 = XtCreateManagedWidget("panel1", formWidgetClass,
					MainBox, NULL, 0);
   Panel2 = XtCreateManagedWidget("panel2", formWidgetClass,
					MainBox, NULL, 0);
   i = 0;
   XtSetArg(args[i], XtNfromHoriz, Panel1);  i++;
   XtSetValues(Panel2, args, i);


/*
 * Create two SimpleButtons: the first is used to exit the application, and
 * the second resets the Lcd1 widget's counter value to 0.
 */
   SimpleButton1 = XtCreateManagedWidget("simpleb1", xcSimpleButtonWidgetClass,
					Panel1, NULL, 0);
   SimpleButton2 = XtCreateManagedWidget("simpleb2", xcSimpleButtonWidgetClass,
					Panel1, NULL, 0);

   i = 0;
   XtSetArg(args[i], XtNfromHoriz, SimpleButton1);  i++;
   XtSetValues(SimpleButton2, args, i);


/*
 * Create two ValueButtons: the first one demonstrates manipulation of
 * hexadecimal values, and the second one manipulates floating point 
 * values.
 */
   VButton1 = XtCreateManagedWidget("vbutton1", xcValueButtonWidgetClass,
                        Panel1, NULL, 0);
   VButton2 = XtCreateManagedWidget("vbutton2", xcValueButtonWidgetClass,
                        Panel1, NULL, 0);
   i = 0;
/* Uncomment these resource settings to test hard-coding of resources. */
/*
   up_bound.fval = 100.00;
   lo_bound.fval = 0.00;
   inc.fval = 0.01;
   test.fval = 0.00;
   XtSetArg(args[i], XcNlabel, "Value1");  i++;
   XtSetArg(args[i], XcNshadeDepth, 2);  i++;
   XtSetArg(args[i], XcNdataType, XcFval);  i++;
   XtSetArg(args[i], XcNdecimals, 2);  i++;
   XtSetArg(args[i], XcNvalueJustify, XcJustifyLeft);  i++;
   XtSetArg(args[i], XcNupperBound, up_bound.lval);  i++;
   XtSetArg(args[i], XcNlowerBound, lo_bound.lval);  i++;
   XtSetArg(args[i], XcNincrement, inc.lval);  i++;
   XtSetArg(args[i], XcNvalue, test.lval);  i++;
*/
   XtSetArg(args[i], XtNfromVert, SimpleButton1);  i++;
   XtSetValues(VButton1, args, i);
   i = 0;
   XtSetArg(args[i], XtNfromVert, SimpleButton2);  i++;
   XtSetArg(args[i], XtNfromHoriz, VButton1);  i++;
   XtSetValues(VButton2, args, i);


/*
 * Create two Sliders: the first one manipulates integer values,
 * and the second one manipulates floating point values.
 */
   Slider1 = XtCreateManagedWidget("slider1", xcSliderWidgetClass,
                        Panel1, NULL, 0);
   Slider2 = XtCreateManagedWidget("slider2", xcSliderWidgetClass,
                        Panel1, NULL, 0);
   i = 0;
   XtSetArg(args[i], XtNfromVert, VButton1);  i++;
   XtSetValues(Slider1, args, i);
   i = 0;
   XtSetArg(args[i], XtNfromVert, Slider1);  i++;
   XtSetValues(Slider2, args, i);


/*
 * Create two ToggleButtons: the first one simply toggles on/off,
 * and the second one is used to enable/disable displaying of Slider1's
 * value in BarGraph1.
 */
   Toggle1 = XtCreateManagedWidget("toggle1", xcToggleButtonWidgetClass,
                        Panel1, NULL, 0);
   Toggle2 = XtCreateManagedWidget("toggle2", xcToggleButtonWidgetClass,
                        Panel1, NULL, 0);
   i = 0;
   XtSetArg(args[i], XtNfromVert, VButton2);  i++;
   XtSetArg(args[i], XtNfromHoriz, Slider1);  i++;
   XtSetValues(Toggle1, args, i);
   i = 0;
   XtSetArg(args[i], XtNfromVert, Toggle1);  i++;
   XtSetArg(args[i], XtNfromHoriz, Slider1);  i++;
   XtSetValues(Toggle2, args, i);


/*
 * Create two BarGraphs: the first one displays Slider1's value if
 * Toggle2 is set to On, and the second one is used to display Lcd1's
 * value.
 */
   BarGraph1 = XtCreateManagedWidget("bargraph1", xcBarGraphWidgetClass,
                        Panel2, NULL, 0);
   BarGraph2 = XtCreateManagedWidget("bargraph2", xcBarGraphWidgetClass,
                        Panel2, NULL, 0);
   i = 0;
   XtSetArg(args[i], XtNfromVert, BarGraph1);  i++;
   XtSetValues(BarGraph2, args, i);


/*
 * Create two LedButtons: the first one represents VButton2's voltage state
 * as it passes through increments of 5.0, and the second one flashes 
 * red every second.
 */
   Led1 = XtCreateManagedWidget("led1", xcLedButtonWidgetClass,
                        Panel2, NULL, 0);
   Led2 = XtCreateManagedWidget("led2", xcLedButtonWidgetClass,
                        Panel2, NULL, 0);
   Led3 = XtCreateManagedWidget("led3", xcLedButtonWidgetClass,
                        Panel2, NULL, 0);
   i = 0;
   XtSetArg(args[i], XtNfromHoriz, BarGraph2);  i++;
   XtSetArg(args[i], XtNfromVert, BarGraph1);  i++;
   XtSetValues(Led1, args, i);
   i = 0;
   XtSetArg(args[i], XtNfromHoriz, BarGraph2);  i++;
   XtSetArg(args[i], XtNfromVert, Led1);  i++;
   XtSetValues(Led2, args, i);
   i = 0;
   XtSetArg(args[i], XtNfromHoriz, BarGraph2);  i++;
   XtSetArg(args[i], XtNfromVert, Led2);  i++;
   XtSetValues(Led3, args, i);


/*
 * Create two Lcds: the first one is a count-up integer counter,
 * and the second one is used to display Slider2's value.
 */
   Lcd1 = XtCreateManagedWidget("lcd1", xcLcdWidgetClass,
                        Panel2, NULL, 0);
   Lcd2 = XtCreateManagedWidget("lcd2", xcLcdWidgetClass,
                        Panel2, NULL, 0);
   i = 0;
   XtSetArg(args[i], XtNfromVert, BarGraph2);  i++;
   XtSetValues(Lcd1, args, i);
   i = 0;
   XtSetArg(args[i], XtNfromVert, Lcd1);  i++;
   XtSetValues(Lcd2, args, i);


/* Now register the callback functions for these widgets. */
   XtAddCallback(SimpleButton1, XcNactivateCallback, Quit, NULL);
   XtAddCallback(SimpleButton2, XcNactivateCallback, Reset_lcd, Lcd1);
   XtAddCallback(VButton1, XcNvalueCallback, Val_callback, NULL);
   XtAddCallback(VButton2, XcNvalueCallback, Set_threshold, Led1);
   XtAddCallback(Led2, XcNupdateCallback, Led2_callback, NULL);
   XtAddCallback(Slider1, XcNvalueCallback, Slider1_callback, BarGraph1);
   XtAddCallback(Slider2, XcNvalueCallback, Slider2_callback, Lcd2);
   XtAddCallback(Lcd1, XcNupdateCallback, Lcd1_callback, BarGraph2);
   XtAddCallback(Led3, XcNupdateCallback, Led3_callback, NULL);



   XtRealizeWidget(TopLevel);

   XtMainLoop();

} 




